/*    Copyright (C) 2006 Robert Kooima                                       */
/*                                                                           */
/*    Varrier Combiner is free software;  you can  redistribute it and/or    */
/*    modify  it under  the terms  of the  GNU General Public License  as    */
/*    published by the Free Software Foundation;  either version 2 of the    */
/*    License, or (at your option) any later version                         */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful, but    */
/*    WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of    */
/*    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU    */
/*    General Public License for more details.                               */

#ifndef VARRIER_COMBINER_H
#define VARRIER_COMBINER_H

#include <GL/gl.h>
#include <GL/glu.h>

/*---------------------------------------------------------------------------*/

class VarrierCombiner
{
  public:
    struct Display
    {
        int viewport_x;
        int viewport_y;
        int viewport_w;
        int viewport_h;

        float quality;

        float screen_BL[3];
        float screen_TL[3];
        float screen_BR[3];

        float pitch;
        float angle;
        float thick;
        float shift;
        float cycle;
    };

    VarrierCombiner(int, int, float, bool=false);
    virtual ~VarrierCombiner();
    void prepare(const struct VarrierCombiner::Display*, int);
    void combine(const struct VarrierCombiner::Display*, const float[3], const float[3]);
    void frustum(const struct VarrierCombiner::Display*, const float[3], float, float);
    void setVerbose(bool);
    void printGLError(const char* msg);

  public:
    static VarrierCombiner* _instance;      ///< NULL as long as no instance has been declared
    static char* vert_txt;
    static char* frag_txt;
    GLuint vert_obj;
    GLuint frag_obj;
    GLuint prog_obj;
    GLuint frame_buf[2];  ///< two buffers for left and right eye
    GLuint color_buf[2];
    GLuint depth_buf[2];
    bool _verbose;

    void check_log(GLhandleARB handle);
    void normalize(float v[3]);
    int check_ext(const char *needle);
    int init_ogl();
    void init_buffers(GLuint *frame, GLuint *color, GLuint *depth, int w, int h);
    void init_program(GLhandleARB *prog, GLhandleARB *vert, GLhandleARB *frag, const char *vert_txt, const char *frag_txt);
    void basis(const struct VarrierCombiner::Display *V, float R[3], float U[3], float N[3]);
    void draw_transform(const struct VarrierCombiner::Display *V, const float P[3]);
};

#endif
